#include "framecpp/config.h"

#include <memory>

#include "framecpp/Common/IOStream.hh"
#include "framecpp/Common/Description.hh"
#include "framecpp/Common/FrameSpec.tcc"

#include "framecpp/Version7/FrMsg.hh"

#include "framecpp/Version8/FrMsg.hh"
#include "framecpp/Version8/FrSE.hh"
#include "framecpp/Version8/FrSH.hh"

#include "framecpp/Version8/PTR_STRUCT.hh"

using namespace FrameCPP::Version_8;
using FrameCPP::Common::Description;
using FrameCPP::Common::FrameSpec;
using FrameCPP::Common::IStream;
using FrameCPP::Common::OStream;


//=======================================================================
// Static
//=======================================================================
static const FrameSpec::Info::frame_object_types s_object_id
= FrameSpec::Info::FSI_FR_MSG;

//=======================================================================
// FrMsg::fr_event_data_type
//=======================================================================

bool FrMsg::fr_msg_data_type::
operator==( const fr_msg_data_type& RHS ) const
{
#define CMP__(X) ( X == RHS.X )

  return ( ( &RHS == this )
	   || ( CMP__( alarm )
		&&  CMP__( message )
		&&  CMP__( severity )
		&& CMP__( GTime )
	   ) )
    ;
#undef CMP__
}

//=======================================================================
// FrMsg
//=======================================================================
FrMsg::
FrMsg( )
  : object_type( s_object_id, StructDescription( ) )
{
}

FrMsg::
FrMsg( const FrMsg& Source )
  : object_type( s_object_id, StructDescription( ) )
{
  m_data.alarm = Source.m_data.alarm;
  m_data.message = Source.m_data.message;
  m_data.severity = Source.m_data.severity;
  m_data.GTime = Source.m_data.GTime;
}

FrMsg::
FrMsg( const std::string& Alarm,
       const std::string& Message,
       INT_4U Severity,
       const GPSTime& GTime )
  : object_type( s_object_id, StructDescription( ) )
{
  m_data.alarm = Alarm;
  m_data.message = Message;
  m_data.severity = Severity;
  m_data.GTime = GTime;
}

FrMsg::
FrMsg( Previous::FrMsg& Source, IStream* Stream )
  : object_type( s_object_id, StructDescription( ) )
{
  m_data.alarm = Source.GetAlarm( );
  m_data.message = Source.GetMessage( );
  m_data.severity = Source.GetSeverity( );
  m_data.GTime = Source.GetGTime( );
  if ( Stream )
  {
    /// \todo Need to replace the next reference
  }
}

FrMsg::
FrMsg( IStream& Stream )
  : object_type( s_object_id, StructDescription( ) )
{
  Stream >> m_data.alarm
	 >> m_data.message
	 >> m_data.severity
	 >> m_data.GTime
    ;
  Stream.Next( this );
}

const std::string& FrMsg::
GetAlarm( ) const
{
   return m_data.alarm;
}


const char* const FrMsg::
ObjectStructName( ) const
{
  return StructName( );
}

const Description* FrMsg::
StructDescription( )
{
  static Description ret;

  if ( ret.size( ) == 0 )
  {

    ret( FrSH( FrMsg::StructName( ), s_object_id,
	       "Msg Data Structure" ) );

    ret( FrSE( "alarm", "STRING",
	       "" ) );
    ret( FrSE( "message", "STRING",
	       "" ) );
    ret( FrSE( "severity", "INT_4U",
	       "" ) );
    ret( FrSE( "GTimeS", "INT_4U",
	       "" ) );
    ret( FrSE( "GTimeN", "INT_4U",
	       "" ) );

    ret( FrSE( "next", PTR_STRUCT::Desc( FrMsg::StructName( ) ),
	       "" ) );

    ret( FrSE( "chkSum", CheckSumDataClass( ), CheckSumDataComment( ) ) );
  }

  return &ret;
}

void FrMsg::
TOCQuery( int InfoClass, ... ) const
{
  using Common::TOCInfo;

  va_list	vl;
  va_start( vl, InfoClass );
  va_end( vl );
}

FrMsg& FrMsg::
Merge( const FrMsg& RHS )
{
  //:TODO: Need to implement Merge routine
  std::string msg( "Merge currently not implemented for " );
  msg += StructName( );

  throw std::domain_error( msg );
  return *this;
}

std::auto_ptr< FrameSpec::Object > FrMsg::
Promote( INT_2U Source,
	 FrameSpec::Object* Obj, Common::IStream* Stream )
{
  return Object::PromoteObject< Previous::FrMsg, FrMsg >
    ( DATA_FORMAT_VERSION, Source, Obj, Stream );
}

bool FrMsg::
operator==( const Common::FrameSpec::Object& Obj ) const
{
  return ( *this == Obj );
}

std::auto_ptr< FrameSpec::Object > FrMsg::
demote( INT_2U Target, IStream* Stream ) const
{
  if ( Target >= DATA_FORMAT_VERSION )
  {
    return std::auto_ptr< FrameSpec::Object >
      ( const_cast< FrMsg* >( this ) );
  }
  try
  {
    //-------------------------------------------------------------------
    // Copy non-reference information
    //-------------------------------------------------------------------
    std::auto_ptr< Previous::FrMsg >
      retval( new Previous::FrMsg( GetAlarm( ),
				   GetMessage( ),
				   GetSeverity( ),
				   GetGTime( )
				   ) )
      ;
    //-------------------------------------------------------------------
    // Modify references
    //-------------------------------------------------------------------
    /// \todo Replace next references
    //-------------------------------------------------------------------
    // Return demoted object
    //-------------------------------------------------------------------
    return std::auto_ptr< FrameSpec::Object >( retval.release( ) );
  }
  catch( ... )
  {
  }
  throw
    Unimplemented( "Object* FrMsg::Demote( Object* Obj ) const",
		   DATA_FORMAT_VERSION, __FILE__, __LINE__ );
}

std::auto_ptr< FrameSpec::Object > FrMsg::
promote( INT_2U Target, Object* Obj, IStream* Stream ) const
{
  return Promote( Target, Obj, Stream );
}

FrameCPP::cmn_streamsize_type FrMsg::
pBytes( const Common::StreamBase& Stream ) const
{
  return
    m_data.alarm.Bytes( )
    + m_data.message.Bytes( )
    + sizeof( m_data.severity )
    + sizeof( INT_4U )		// GTime Seconds;
    + sizeof( INT_4U )		// GTime Nanoeconds;
    + Stream.PtrStructBytes( )	// next
    ;
}

FrMsg* FrMsg::
pCreate( IStream& Stream ) const
{
  return new FrMsg( Stream );
}

void FrMsg::
pWrite( OStream& Stream ) const
{
  Stream << m_data.alarm
	 << m_data.message
	 << m_data.severity
	 << m_data.GTime
    ;
  WriteNext( Stream );
}

