#ifndef FrameCPP_VERSION_8_FrTOC_HH
#define FrameCPP_VERSION_8_FrTOC_HH

#include <map>
#include <vector>

#include "framecpp/Common/FrameSpec.hh"
#include "framecpp/Common/FrTOC.hh"

#include "framecpp/Version7/FrTOC.hh"

#include "framecpp/Version8/FrameSpec.hh"
#include "framecpp/Version8/FrTOCData.hh"
#include "framecpp/Version8/FrTOCStatData.hh"
#include "framecpp/Version8/FrTOCAdcData.hh"

#include "framecpp/Version8/FrTOCProcData.hh"
#include "framecpp/Version8/FrTOCSimData.hh"
#include "framecpp/Version8/FrTOCSerData.hh"
#include "framecpp/Version8/FrTOCSummary.hh"
#include "framecpp/Version8/FrTOCEvent.hh"
#include "framecpp/Version8/FrTOCSimEvent.hh"


namespace FrameCPP
{
  namespace Version_8
  {

    //===================================================================
    //===================================================================

    class FrTOC
      : public ObjectWithChecksum< Common::FrTOC >,
        public FrTOCData,
	public FrTOCStatData,
	public FrTOCAdcData,
	public FrTOCProcData,
	public FrTOCSimData,
	public FrTOCSerData,
	public FrTOCSummary,
	public FrTOCEvent,
	public FrTOCSimEvent
    {
    public:
      typedef INT_8U				position_type;
      typedef std::vector< position_type >	position_list_type;

      class StatTypeKey {
      public:
	std::string	s_name;
	std::string	s_detector;

	StatTypeKey( const std::string& Name,
		     const std::string& Detector );
	bool operator<( const StatTypeKey& RHS ) const;
	bool operator==( const StatTypeKey& RHS ) const;
	bool operator!=( const StatTypeKey& RHS ) const;
      };

      struct StatType_type
      {
	std::vector< INT_4U >	tStart;
	std::vector< INT_4U >	tEnd;
	std::vector< INT_4U >	version;
	position_list_type	positionStat;
      };

      class StatTypeKeyHash
      {
      public:
	size_t operator()( const StatTypeKey& Key ) const
	{
	  General::hash< const char* > h;

	  std::string key( Key.s_name );
	  key += Key.s_detector;
	  return h( key.c_str( ) );
	}
      };

      typedef General::unordered_map< StatTypeKey,
				      StatType_type,
				      StatTypeKeyHash >
      MapStatType_type;

      static const INT_4U	NO_DATA_AVAILABLE;

      FrTOC( );

      static const char* const StructName( );

      static const Common::Description* StructDescription( );

      virtual void IndexObject( const Common::FrameSpec::Object* Object,
				std::streampos Position );

      virtual FrTOC* Create( ) const;

      virtual void FrStatDataQuery( const std::string& NamePattern,
				    const General::GPSTime& StartTime,
				    const General::GPSTime& EndTime,
				    const INT_4U Version,
				    Common::FrStatData::Query& Result ) const;

      virtual const char* const ObjectStructName( ) const;

      /// \brief comparison operator
      virtual bool operator==( const Common::FrameSpec::Object& Obj ) const;

      const MapStatType_type&		GetStatType( ) const;

    protected:
      //-----------------------------------------------------------------
      /// \brief Down grade an object
      //-----------------------------------------------------------------
      virtual std::auto_ptr< Object >
      demote( INT_2U Target, Common::IStream* Stream ) const;

      //-----------------------------------------------------------------
      /// \brief Upgrade an object
      //-----------------------------------------------------------------
      virtual std::auto_ptr< Object >
      promote( INT_2U Target, Object* Obj, Common::IStream* Stream ) const;

      //-----------------------------------------------------------------
      /// \brief Number of bytes needed to write this structure
      //-----------------------------------------------------------------
      virtual Common::FrameSpec::size_type
      pBytes( const Common::StreamBase& Stream ) const;

      virtual FrTOC* pCreate( Common::IStream& Stream ) const;

      //-----------------------------------------------------------------
      /// \brief Write the structure to the stream
      //-----------------------------------------------------------------
      virtual void pWrite( Common::OStream& Stream ) const;
 
    protected:
      //-----------------------------------------------------------------
      // Interface routines
      //-----------------------------------------------------------------
      virtual INT_4U nFrame( ) const;

      virtual const cmn_dt_container_type& dt( ) const;
      virtual const cmn_GTimeS_container_type& GTimeS( ) const;
      virtual const cmn_GTimeN_container_type& GTimeN( ) const;

      virtual INT_8U positionH( INT_4U FrameIndex ) const;

      virtual const cmn_name_container_type& nameADC( ) const;

      virtual INT_8U positionADC( INT_4U FrameIndex,
				  const std::string& Channel ) const;
      virtual INT_8U positionADC( INT_4U FrameIndex,
				  INT_4U Channel ) const;

      virtual INT_8U positionEvent( INT_4U FrameIndex,
				    const std::string& Event ) const;

      virtual const cmn_name_container_type& nameProc( ) const;

      virtual INT_8U positionProc( INT_4U FrameIndex,
				   const std::string& Channel ) const;
      virtual INT_8U positionProc( INT_4U FrameIndex,
				   INT_4U Channel ) const;

      virtual const cmn_name_container_type& nameSer( ) const;

      virtual INT_8U positionSer( INT_4U FrameIndex,
				  const std::string& Channel ) const;

      virtual const cmn_name_container_type& nameSim( ) const;

      virtual INT_8U positionSim( INT_4U FrameIndex,
				  const std::string& Channel ) const;

      virtual INT_8U positionSimEvent( INT_4U FrameIndex,
				       const std::string& SimEvent ) const;

      virtual INT_4U nSH( ) const;
      virtual INT_2U SHid( INT_4U Offset ) const;
      virtual const std::string& SHname( INT_4U Offset ) const;

    private:
      //-----------------------------------------------------------------
      //-----------------------------------------------------------------
      typedef ObjectWithChecksum< Common::FrTOC > object_type;

      using Common::FrameSpec::Object::Create;

      MapStatType_type		m_StatType;

      FrTOC( Common::IStream& Stream );

      template< typename ChannelType >
      INT_8U position_adc( INT_4U FrameIndex, ChannelType Channel ) const;

      template< typename ChannelType >
      INT_8U position_proc( INT_4U FrameIndex, ChannelType Channel ) const;

   };

    inline const char* const FrTOC::
    StructName( )
    {
      static const CHAR* class_name( "FrTOC" );
      return class_name;
    }

    inline const FrTOC::MapStatType_type& FrTOC::
    GetStatType( ) const
    {
      return m_StatType;
    }

  } // namespace - Version_8
} // namespace - FrameCPP

#endif /* FrameCPP_VERSION_8_FrTOC_HH */
