/* -*- mode: C++; c-basic-offset: 2; -*- */
#if HAVE_CONFIG_H
#include "framecpp/config.h"
#endif /* HAVE_CONFIG_H */

#include <string.h>
#include <unistd.h>

#include <iostream>

#include "general/AtExit.hh"

#include "framecpp/Common/FrameSpec.hh"
#include "framecpp/Common/IOStream.hh"
#include "framecpp/Common/FrameStream.hh"
#include "framecpp/Common/FrTOC.hh"

#include "framecpp/Version4/FrTOC.hh"
#include "framecpp/Version6/FrTOC.hh"

#include "framecpp/FrameCPP.hh"

#include "toc.hh"

using namespace FrameCPP::Common;

using std::filebuf;

char* Program = 0;

void usage( int ExitCode )
{
  std::cerr << "Usage: "
	    << Program
	    << " [-h]"
	    << " [--h]"
	    << " frame_file [ frame_file ... ]" << std::endl
	    << "\t-h  - Print this usage message" << std::endl
	    << "\t--h - Print this usage message" << std::endl
    ;
  exit( ExitCode );
}

int
main(int argc, char* argv[])
try
{
  /* Determine program name (strip filesystem path) */
  for ( Program = argv [0] + strlen (argv [0]) - 1;
        Program != argv [0] && *Program != '/';
	Program--)
    ;
  
  if (*Program == '/')
  {
    Program++;
  }

  //---------------------------------------------------------------------
  // Parse arguments
  //---------------------------------------------------------------------
  int		args_parsed;
  
  for ( args_parsed = 1; args_parsed < argc; ++args_parsed )
  {
    char* cur_arg = argv[ args_parsed ];
	
    if ( strlen( cur_arg ) >= 2 )
    {
      if ( ( strncasecmp( cur_arg, "-h", 2 ) == 0 )
	   || ( strncasecmp( cur_arg, "--h", 3 ) == 0 ) )
      {
	usage( 0 );
      }
      else if ( *cur_arg == '-' )
      {
	std::cerr << "ERROR: Unknown option: " << cur_arg
		  << std::endl;
	usage( 1 );
      }
      else
      {
	break;
      }
    }
  }
  //---------------------------------------------------------------------
  // Process files
  //---------------------------------------------------------------------
  for ( ; args_parsed < argc; ++args_parsed )
  {
    char* filename = argv[ args_parsed ];
    //-------------------------------------------------------------------
    // Creation of the frame structure by reading of frame file
    //-------------------------------------------------------------------
    FrameBuffer< filebuf >*
      ibuf( new FrameBuffer< filebuf >( std::ios::in ) );
    
    ibuf->open( filename, std::ios::in | std::ios::binary );

    IFrameStream	ifs( ibuf, 0 );

    std::cout << "Frame file: " << filename << std::endl
	      << "Frame Specification: " << ifs.Version( ) << std::endl
      ;
    const FrTOC* toc( ifs.GetTOC( ) );

    dump( toc );
  }
  //---------------------------------------------------------------------
  // Exit
  //---------------------------------------------------------------------
  General::AtExit::Cleanup( );
}
catch( ... )
{
  General::AtExit::Cleanup( );
}
